
#include "mp4.h"
#include <locale.h>
#include <langinfo.h>
#include <iconv.h>

static char tempStr[ 1024];
static char titleStr[ 1024];
static char mp4fileStr[ 1024];

int main (int argc, const char * argv[]) {
    char* usageString = 
    " <-o|-c|-d|-dd> <mp4-file>\n"
    "	-p do optimize -p mp4file [-o outfile] [-t title]\n"
    "	-c check format\n"
    "	-d dump (-dd verbose dump)\n"
    "\nmp4psp 0.9.1 (C)2004 Hetima Computer.\nbased on mpeg4ip.\n";

    bool doCheck = false;
    bool doOptimize = false;
    bool doDump = false;
    
    bool renameTemp = true;
    bool verbose = false;

    const char* mp4FileName = NULL;
    const char* tempFile = NULL;
    const char* title = NULL;

    // begin processing command line
    const char*   ProgName;

    //ProgName = argv[0];
    ProgName="mp4psp";

    // check that we have at least one non-option argument
    if (argc <= 2) {
        fprintf(stderr, "usage: %s %s", ProgName, usageString);
        return 0;
    }

    if (!strcmp(argv[1], "-p")) {
        doOptimize = true;
    }else if (!strcmp(argv[1], "-c")) {
        doCheck = true;
    }else if (!strcmp(argv[1], "-d")) {
        doDump = true;
    }else if (!strcmp(argv[1], "-dd")) {
        doDump = true;
        verbose = true;
    }

    mp4FileName = argv[2];
    
    if(argv[3] && argv[4]){
        if(!strcmp(argv[3], "-o")){
            tempFile=argv[4];
            renameTemp=false;
        }else if(!strcmp(argv[3], "-t")){
            title=argv[4];
        }
        if(argv[5] && argv[6]){
            if(!strcmp(argv[5], "-o")){
                tempFile=argv[6];
                renameTemp=false;
            }else if(!strcmp(argv[5], "-t")){
                title=argv[6];
            }
        }

    }

    // end processing of command line

    // test if mp4 file exists
    {
            bool mp4FileExists;

            mp4FileExists = (access(mp4FileName, F_OK) == 0);
            if (!mp4FileExists) {
                fprintf(stderr, "%s: file doesn't exist\n", ProgName);
                exit(0);
            }
    }

    if (doCheck) {
            // generate uuid atom
            char* info;

            info = MP4FileInfo(mp4FileName, MP4_INVALID_TRACK_ID);
            if (!info) {
                fprintf(stderr,
                        "%s: this file seems NOT valid\n", mp4FileName);
                exit(0);
            }else{
                fprintf(stderr,
                        "%s: this file seems valid format\n", mp4FileName);
            }

            //fputs(info, stdout);
            free(info);

    }

    if (doDump) {
        MP4FileHandle mp4File;

        mp4File = MP4Read(mp4FileName, MP4_DETAILS_ERROR);
        if (!mp4File) {
            exit(1);
        }
        if(verbose)MP4SetVerbosity(mp4File, MP4_DETAILS_ALL);

        MP4Dump(mp4File, stdout, 0);
        MP4Close(mp4File);
        exit(0);
    }

    setlocale( LC_ALL, "");

    if (doOptimize) {
        iconv_t cd;
        size_t insize;
        const char* bytes=NULL;
        int length;
        
        //file path
        strcpy( mp4fileStr, mp4FileName);

        if(tempFile && !strcmp(tempFile, mp4FileName) ){
            tempFile = NULL;
            renameTemp = true;
        }
        
        if(tempFile == NULL){
            strcpy( tempStr, mp4fileStr);
            strcat( tempStr, ".tempfile");
            tempFile = tempStr;
        }
        
        //title
        if(title==NULL){
            title = mp4fileStr;
        }

        bytes = titleStr;
        insize = strlen( title);
        length = sizeof( titleStr);

        //cd = iconv_open( "UTF-16BE", nl_langinfo(CODESET));
        cd = iconv_open( "UTF-16BE", "EUC-JP" );
        iconv( cd, &title, &insize, &bytes, &length);
        iconv_close( cd);

        length = bytes - titleStr;
        bytes -= length;

        if (MP4OptimizeForPSP(mp4FileName, tempFile, bytes, length, 0)){
            if(renameTemp) rename(tempFile, mp4FileName);
            // mp4 library should have printed a message
            //exit(0);
        }
    }

    return 0;
}
